from django.conf import settings
from django.http import Http404, HttpResponse, HttpResponsePermanentRedirect
from django.core.xheaders import populate_xheaders
from django.shortcuts import get_object_or_404
from django.template import loader, RequestContext
from django.utils.safestring import mark_safe
from django.views.decorators.csrf import csrf_protect

from .models import Page


# This view is called from PageFallbackMiddleware.process_response
# when a 404 is raised, which often means CsrfViewMiddleware.process_view
# has not been called even if CsrfViewMiddleware is installed. So we need
# to use @csrf_protect, in case the template needs {% csrf_token %}.
# However, we can't just wrap this view; if no matching page exists,
# or a redirect is required for authentication, the 404 needs to be returned
# without any CSRF checks. Therefore, we only
# CSRF protect the internal implementation.
def page(request, path):
    """Public interface to the page view.

    Models: `paginas.Page`
    Templates: Uses the template defined by the ``template``.
    Context:
        page
            `paginas.Page` object

    """
    if not path.startswith('/'):
        path = '/' + path
    try:
        page = get_object_or_404(Page, path__exact=path)
    except Http404:
        if not path.endswith('/') and settings.APPEND_SLASH:
            path += '/'
            page = get_object_or_404(Page, path__exact=path)
            return HttpResponsePermanentRedirect('%s/' % request.path)
        else:
            raise
    return render_page(request, page)


@csrf_protect
def render_page(request, page):
    """Internal interface to the page view.
    """
    # To avoid having to always use the "|safe" filter in page templates,
    # mark the title and content as already safe (since they are raw HTML
    # content in the first place).
    page.title = mark_safe(page.title)
    page.content = mark_safe(page.content)

    context = RequestContext(request, {'page': page})
    template = loader.get_template(page.template.name)
    response = HttpResponse(template.render(context))
    populate_xheaders(request, response, Page, page.id)
    return response
